# Generated by Django 4.0.4 on 2022-05-10 14:09

from django.db import migrations

import audoma.django.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ("audoma_api", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="examplemodel",
            name="json",
            field=audoma.django.db.fields.JSONField(),
        ),
        migrations.AlterField(
            model_name="examplemodel",
            name="mac_adress",
            field=audoma.django.db.fields.MACAddressField(integer=True),
        ),
        migrations.AlterField(
            model_name="examplemodel",
            name="money_currency",
            field=audoma.django.db.fields.CurrencyField(
                choices=[
                    ("AUD", "Australian Dollar"),
                    ("GBP", "British Pound"),
                    ("CNY", "Chinese Yuan"),
                    ("EUR", "Euro"),
                    ("INR", "Indian Rupee"),
                    ("JPY", "Japanese Yen"),
                    ("NZD", "New Zealand Dollar"),
                    ("CHF", "Swiss Franc"),
                    ("USD", "US Dollar"),
                ],
                default="XYZ",
                max_length=3,
            ),
        ),
        migrations.AlterField(
            model_name="exampleperson",
            name="savings_currency",
            field=audoma.django.db.fields.CurrencyField(
                choices=[
                    ("AUD", "Australian Dollar"),
                    ("GBP", "British Pound"),
                    ("CNY", "Chinese Yuan"),
                    ("EUR", "Euro"),
                    ("INR", "Indian Rupee"),
                    ("JPY", "Japanese Yen"),
                    ("NZD", "New Zealand Dollar"),
                    ("CHF", "Swiss Franc"),
                    ("USD", "US Dollar"),
                ],
                default="PLN",
                max_length=3,
            ),
        ),
    ]
