# Generated by Django 3.2.12 on 2022-04-21 12:41

from django.db import (
    migrations,
    models,
)

import audoma.django.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ExampleFileModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("file_field", models.FileField(upload_to="")),
                ("name", audoma.django.db.fields.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name="ExampleModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("char_field", audoma.django.db.fields.CharField(max_length=255)),
                (
                    "phone_number",
                    audoma.django.db.fields.PhoneNumberField(
                        max_length=128, region=None
                    ),
                ),
                (
                    "phone_number_example",
                    audoma.django.db.fields.PhoneNumberField(
                        max_length=128, region=None
                    ),
                ),
                ("email", audoma.django.db.fields.EmailField(max_length=254)),
                ("text_field", audoma.django.db.fields.TextField()),
                ("url", audoma.django.db.fields.URLField()),
                ("boolean", audoma.django.db.fields.BooleanField()),
                ("nullboolean", audoma.django.db.fields.BooleanField(null=True)),
                ("mac_adress", audoma.django.db.fields.MACAddressField(integer=True)),
                ("slug", audoma.django.db.fields.SlugField()),
                ("uuid", audoma.django.db.fields.UUIDField()),
                ("ip_address", audoma.django.db.fields.GenericIPAddressField()),
                ("integer", audoma.django.db.fields.IntegerField()),
                ("_float", audoma.django.db.fields.FloatField()),
                (
                    "decimal",
                    audoma.django.db.fields.DecimalField(
                        decimal_places=2, max_digits=10
                    ),
                ),
                ("datetime", audoma.django.db.fields.DateTimeField()),
                ("date", audoma.django.db.fields.DateField()),
                ("time", audoma.django.db.fields.TimeField()),
                ("duration", audoma.django.db.fields.DurationField()),
                (
                    "choices",
                    audoma.django.db.fields.IntegerField(
                        choices=[(1, "example 1"), (2, "example 2"), (3, "example 3")]
                    ),
                ),
                ("json", audoma.django.db.fields.JSONField(default=dict)),
                (
                    "money_currency",
                    audoma.django.db.fields.CurrencyField(
                        choices=[("EUR", "Euro"), ("USD", "US Dollar")],
                        default="XYZ",
                        max_length=3,
                    ),
                ),
                (
                    "money",
                    audoma.django.db.fields.MoneyField(decimal_places=2, max_digits=10),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ExamplePerson",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "first_name",
                    audoma.django.db.fields.CharField(max_length=225, default="Adam"),
                ),
                ("last_name", audoma.django.db.fields.CharField(max_length=255)),
                ("age", audoma.django.db.fields.IntegerField()),
                ("email", audoma.django.db.fields.EmailField(max_length=254)),
                ("birth_country", audoma.django.db.fields.CharField(max_length=255)),
                ("residence_city", audoma.django.db.fields.CharField(max_length=255)),
                ("has_valid_account", audoma.django.db.fields.BooleanField()),
                ("ip_address", audoma.django.db.fields.GenericIPAddressField()),
                (
                    "savings_currency",
                    audoma.django.db.fields.CurrencyField(
                        choices=[("EUR", "Euro"), ("USD", "US Dollar")],
                        default="PLN",
                        max_length=3,
                    ),
                ),
                (
                    "savings",
                    audoma.django.db.fields.MoneyField(
                        decimal_places=2, default_currency="PLN", max_digits=14
                    ),
                ),
                (
                    "phone_number",
                    audoma.django.db.fields.PhoneNumberField(
                        max_length=128, region="IT"
                    ),
                ),
            ],
        ),
    ]
