# Generated by Django 4.0.4 on 2022-05-26 14:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("audoma_api", "0002_alter_examplemodel_json_and_more"),
        ("audoma_api", "0002_examplesimplemodel_alter_examplemodel_json_and_more"),
    ]

    operations = []
